/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 */

 /*
 *
 * @author    Tanja Jovanovic
 * @version   2.0  07.01.2003.
 *
 */


package org.enhydra.dods.cache;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 
 * LRUCache class implements LRU cache for storing DO objects.
 *
 */
public class LRUCache extends LinkedHashMap  {

    /**
     * Maximal number of entries in LRUCache.
     * 1024 is default value of this variable.
     *
     */
    protected int maxEntries = 1024;
    
    /**
     * Constructs an empty access-ordered <tt>LRUCache</tt> instance with the 
     * specified initial capacity, load factor and maximal number of
     * entries.
     *
     * @param  initialCapacity the initial capacity.
     * @param  loadFactor the load factor.
     * @param  maxEnt maximal number of entries in LRUCache.
     */
    public LRUCache (int initialCapacity, float loadFactor, int maxEnt) {
        super(initialCapacity, loadFactor, true);
        maxEntries = maxEnt;
    }
    
    /**
     * Constructs an empty access-ordered <tt>LRUCache</tt> instance with the
     * specified initial capacity and load factor.
     *
     * @param  initialCapacity the initial capacity.
     * @param  loadFactor the load factor.
     */
    public LRUCache (int initialCapacity, float loadFactor) {
        super(initialCapacity, loadFactor, true);
    }

   
    /**
     * Constructs an empty access-ordered <tt>LRUCache</tt> instance with a 
     * default capacity (16) and load factor (0.75) and with maximal number of 
     * entries.
     *
     * @param  maxEnt maximal number of entries in LRUCache.
     */
    public LRUCache(int maxEnt) {
        super(16, (float)0.75, true);
        maxEntries = maxEnt;
    }
    
    /**
     * Constructs an empty access-ordered <tt>LRUCache</tt> instance with a 
     * default capacity (16) and load factor (0.75).
     */
    public LRUCache() {
        super(16, (float)0.75, true);
    }

    /**
     * Constructs an access-ordered <tt>LRUCache</tt> instance with the same 
     * mappings as the specified map. The <tt>LRUCache</tt> instance is created
     * with maximal number of entries, default load factor (0.75) and an initial
     * capacity sufficient to hold the mappings in the specified map.
     *
     * @param  m the map whose mappings are to be placed in this map.
     * @param  maxEnt maximal number of entries in LRUCache.
     */
    public LRUCache(Map m, int maxEnt) {
        super(16, (float)0.75, true);
        maxEntries = maxEnt;
        putAll(m);
    }
    
    /**
     * Constructs an access-ordered <tt>LRUCache</tt> instance with the same 
     * mappings as the specified map. The <tt>LRUCache</tt> instance is created
     * with default load factor (0.75) and an initial capacity sufficient to 
     * hold the mappings in the specified map.
     *
     * @param m the map whose mappings are to be placed in this map.
     */
    public LRUCache(Map m) {
        super(16, (float)0.75, true);
        putAll(m);
    }

    /**
     * Adds a pair (key, value) to the cache. If key already exists, the method 
     * returns previous value asociated with this key. If maximum number of 
     * entries is achieved, the eldest entry will be removed and returned.
     *
     * @param  key key asociated with the value.
     * @param  value value that will be added to the cache.
     * @return previous value asociated with this key, if the key already 
     * exists, or the eldest entry which is removed, if maximum number of 
     * entries is achieved, otherwise null. 
     */
    public Object add(Object key, Object value) {
        Object ret = put(key, value);
        if (ret == null) {
	        if (maxEntries < size() && maxEntries > 0) {
	            Iterator iter = keySet().iterator();
	            Object rem = iter.next();
	            ret = remove(rem);
	        }
        }
	    return ret;
    }

    /**
     * Returns maximal number of entries in LRUCache.
     *
     * @return maximal number of entries in LRUCache.
     */
    public int getMaxEntries () {
        return maxEntries;
    }
    
    /**
     * Sets maximal number of entries in LRUCache.
     *
     * @param max new value of maximal number of entries in LRU Cache.
     */
    public void setMaxEntries (int max) {
        if (max < size() && max > 0) {
            Iterator iter = (new HashSet(keySet())).iterator();
            int dif = size() - max;
            for (int i = 0; i < dif; i++) 
                remove(iter.next());
        }
        maxEntries = max;
    }

    /**
     * For debug reason.
     */
    public String toString() {
        Object[] v = entrySet().toArray();
        String ret = "LRU content:\n";
        for (int i = 0; i < v.length; i++) {
            Map.Entry map = (Map.Entry)v[i];
            ret += i+". key = "+map.getKey()+"  value = "+map.getValue()+"\n";
        } 
        return ret;
    }
}
