 /*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 */

 /*
 *
 * @author    Tanja Jovanovic
 * @version   1.0  23.04.2003.
 *
 */


package org.enhydra.dods.cache;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;

import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.dods.builder.generator.dataobject.GenericDO;

/**
 *
 * This class is for storing one query and its necessary data, for query array.
 *
 */
public class QueryCacheItemImpl implements QueryCacheItem {

    /**
     * Query id: query in the form of string.
     */

    public String queryId;

    /**
     * Array of data object IDs which are results of the query.
     */

    public LinkedHashSet OIds;

    /**
     * Time needed for query execution.
     */

    public int time;

    /**
     * Array of conditions declared in WHERE part of the query
     * (array of org.enhydra.dods.cache.Condition objects).
     */

    public ArrayList conds;

    /**
     * Database of the query.
     */

    public String originDatabase;

    /**
     * Constructor (String).
     *
     * @param  origDb query database
     */
    public QueryCacheItemImpl (String origDb) {
        OIds = new LinkedHashSet();
        this.time = 0;
        conds = new ArrayList();
        originDatabase = origDb;
    }

    /**
     * Constructor (String, LinkedHashSet, int, ArrayList, String).
     *
     * @param  qId query id
     * @param  dataObjects query results
     * @param  time query execution time
     * @param  conditions conditions (WHERE part of the query)
     * @param  origDb query database
     */
    public QueryCacheItemImpl (String qId, LinkedHashSet OIds, int time, ArrayList conditions, String origDb) {
        queryId = qId;
        this.OIds = OIds;
        this.time = time;
        conds = conditions;
        originDatabase = origDb;
    }

    /**
     * Returns query id.
     *
     * @return query id
     */
    public String getQueryId () {
        return queryId;
    }

    /**
     * Sets unique representation of query object.
     *
     * @param queryId <code>String</code> that unique represent query.
     */
    public void setQueryId(String queryId) {
        this.queryId = queryId;
    }

    /**
     * Returns OIds (array of data objects which are the results of the query).
     *
     * @return query array of data objects which are the results of the query
     */
    public HashSet getOIds () {
        return OIds;
    }

    /**
     * Sets OIds (array of data objects which are the results of the query).
     *
     * @parma OIds query array of data objects which are the results of the query.
     */
    public void setOIds (LinkedHashSet OIds) {
        this.OIds = OIds;
    }

    /**
     * Returns time needed for query execution.
     *
     * @return time needed for query execution
     */
    public int getTime () {
        return time;
    }

    /**
     * Sets time needed for query execution.
     *
     * @param time time needed for query execution
     */
    public void setTime (int time) {
        this.time = time;
    }

    /**
     * Returns array of conditions conds.
     *
     * @return array of conditions
     */
    public ArrayList getConds () {
        return conds;
    }

    /**
     * Sets array of conditions.
     *
     * @param conds array of conditions
     */
    public void setConds (ArrayList conds) {
        this.conds = conds;
    }

    /**
     * Add condition to query.
     *
     * @param cond <code>Condition</code> that will be added to query.
     */
    public void addCond (Condition cond) {
        conds.add(cond);
    }

    /**
     * Returns query database.
     *
     * @return query database
     */
    public String getOriginDatabase () {
        return originDatabase;
    }

    /**
     * Checks whether data object obj satisfies conditions of this query.
     *
     * @param  obj dataobject for which are checked conditions of this query
     * @return true if data object obj satisfies conditions of this query,
     * otherwise false.
     */
    public boolean checkConditions (GenericDO obj) {
        for (int i = 0; i < conds.size(); i++){
            if (!obj.compareCond((Condition)conds.get(i)))
                return false;
        }
        return true;
    }

    /**
     * Inserts data object obj (or updates it if already exists) in array OIds,
     * if it satisfies this query.
     *
     * @param obj data object which may be inserted (or updated) in array OIds
     *
     */
    public void update (GenericDO obj) {
        if (obj.getOriginDatabase().equals(originDatabase)) {
            try {
                String key = obj.getHandle();
                if (checkConditions(obj)) {
                    OIds.add(key);
                } else {
                    OIds.remove(key);
                }
            } catch (DatabaseManagerException e){
            }
        }
    }

    /**
     * Removes data object obj from array OIds, if present.
     *
     * @param obj data object which will be removed from array OIds
     */
    public void delete (GenericDO obj) {
        if (obj.getOriginDatabase().equals(originDatabase)) {
            try {
                String key = obj.getHandle();
                OIds.remove(key);
            } catch (DatabaseManagerException e){
            }
        }
    }

    /**
     * Adds data object obj to array OIds.
     *
     * @param obj data object which will be added to array OIds
     */
    public void add (GenericDO obj) {
        try {
            String key = obj.getHandle();
            OIds.add(key);
        } catch (DatabaseManagerException e){
        }
    }

	/**
	 * Shows content of this class.
	 * Can be used for debugging.
	 */
	public String toString() {
		StringBuffer ret = new StringBuffer();
        GenericDO DO = null;
		ret.append("\n QueryCacheItemImpl: ");
		ret.append("\n queryId: " + queryId);
		ret.append("\n OIds : " + OIds);
        if (OIds != null)
            for (Iterator iter = getOIds().iterator(); iter.hasNext(); )
	            ret.append(" " + iter.next());
		ret.append("\n time : " + time);
		ret.append("\n conds : " + conds);
		ret.append("\n originDatabase : " + originDatabase);
        return ret.toString();
 	}
}
